package ru.crystals.pos.api.ext.loyal.dto;

import ru.crystals.pos.spi.receipt.BonusBalance;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

/**
 * Результат начисления бонусов/фишек плагином лояльности.<br>
 * Для плагина лояльности служит способом уведомить кассу о начислении бонусов на карту в результате расчета лояльности.<br>
 * Полученная информация может отображаться на сервере в Опердне или выгружаться в ERP.
 */
public class BonusAccrualResult {
    /**
     * Начисления бонусов/фишек на данную карту.
     */
    private List<BonusAccrual> bonusAccruals = new ArrayList<>();

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link BonusAccrualResult}.
     */
    public BonusAccrualResult() {
        // Nop
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link BonusAccrualResult}.
     * @param transactionId идентификатор транзакции начисления бонусов.
     * @param amount величина начисленных бонусов. Должно быть ненулевым неотрицательным значением.
     */
    public BonusAccrualResult(String transactionId, BigDecimal amount) {
        bonusAccruals.add(new BonusAccrual(transactionId, null, amount));
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link BonusAccrualResult}.
     * @param transactionId идентификатор транзакции начисления бонусов.
     * @param balance баланс, на который предполагается начисление.
     * @param amount величина начисленных бонусов. Должно быть ненулевым неотрицательным значением.
     */
    public BonusAccrualResult(String transactionId, BonusBalance balance, BigDecimal amount) {
        bonusAccruals.add(new BonusAccrual(transactionId, balance, amount));
    }

    /**
     * Получает идентификатор транзакции, которая начислила на счет карты бонусы.
     * @return идентификатор транзакции, которая начислила на счет карты бонусы.
     */
    public String getTransactionId() {
        return bonusAccruals.isEmpty() ? null : bonusAccruals.get(0).getTransactionId();
    }

    /**
     * Устанавливает идентификатор транзакции, которая начислила на счет карты бонусы.
     * @param transactionId идентификатор транзакции, которая начислила на счет карты бонусы.
     */
    public void setTransactionId(String transactionId) {
        if(bonusAccruals.isEmpty()) {
            bonusAccruals.add(new BonusAccrual());
        }
        bonusAccruals.get(0).setTransactionId(transactionId);
    }

    /**
     * Возвращает величину начисленных на счет карты бонусов.
     * @return величина начисленных на счет карты бонусов.
     */
    public BigDecimal getAmount() {
        return bonusAccruals.isEmpty() ? null : bonusAccruals.get(0).getAmount();
    }

    /**
     * Устанавливает величину начисленных на счет карты бонусов.
     * @param amount величина начисленных на счет карты бонусов.
     */
    public void setAmount(BigDecimal amount) {
        if(bonusAccruals.isEmpty()) {
            bonusAccruals.add(new BonusAccrual());
        }
        bonusAccruals.get(0).setAmount(amount);
    }

    /**
     * @return начисления бонусов/фишек на данную карту
     */
    public List<BonusAccrual> getBonusAccruals() {
        return bonusAccruals;
    }

    @Override
    public String toString() {
        return "BonusAccrualResult{" +
                "bonusAccruals=" + bonusAccruals +
                '}';
    }
}
