package ru.crystals.pos.spi.equipment;

import java.time.Duration;
import ru.crystals.pos.api.comm.CommunicationMessage;
import ru.crystals.pos.api.comm.table.Table;

/**
 * Структура для отображения сообщения на экране покупателя.
 * @since 0.0.50
 */
public class CustomerDisplayMessage {
    private Duration displayDuration;
    private CommunicationMessage message;
    private Table tableMessage;

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link CustomerDisplayMessage}.
     * @param message текстовое сообщение, которое требуется отобразить на экране покупателя.
     */
    public CustomerDisplayMessage(String message) {
        this(message, null);
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link CustomerDisplayMessage}.
     * @param message текстовое сообщение, которое требуется отобразить на экране покупателя.
     * @param duration период времени, в течение которого сообщение следует отображать. Не гарантируется, что сообщение будет отображаться ровно
     *                 этот период времени, поскольку оно может быть замещено иным сообщением.
     */
    public CustomerDisplayMessage(String message, Duration duration) {
        this(new CommunicationMessage(null, null, message, null), duration);
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link CustomerDisplayMessage}.
     * @param message расширенный тип сообщения, который требуется отбразить на дисплее покупателя.
     *                Не все дисплеи покупателя способны отображать этот тип сообщений.
     */
    public CustomerDisplayMessage(CommunicationMessage message) {
        this(message, null);
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link CustomerDisplayMessage}.
     * @param message расширенный тип сообщения, который требуется отобразить на дисплее покупателя.
     *                Не все дисплеи покупателя способны отображать этот тип сообщений.
     * @param duration период времени, в течение которого сообщение следует отображать. Не гарантируется, что сообщение будет отображаться
     *                 ровно этот период времени, поскольку оно может быть замещено иным сообщением.
     */
    public CustomerDisplayMessage(CommunicationMessage message, Duration duration) {
        this.message = message;
        this.displayDuration = duration;
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link CustomerDisplayMessage}.
     * @param tableMessage табличное расширенное сообщение, предназначено для отображения блоков текста в виде таблицы. Не все
     *                     дисплеи покупателя поддерживают такой тип сообщения.
     */
    public CustomerDisplayMessage(Table tableMessage) {
        this(tableMessage, null);
    }

    /**
     * Конструктор класса. Создаёт новый экземпляр класса {@link CustomerDisplayMessage}.
     * @param tableMessage табличное расширенное сообщение, предназначено для отображения блоков текста в виде таблицы.
     *                     Не все дисплеи покупателя поддерживают такой тип сообщения.
     * @param duration период времени, в течение которого сообщение следует отображать. Не гарантируется, что сообщение будет отображаться
     *                 ровно этот период времени, поскольку оно может быть замещено иным сообщением.
     */
    public CustomerDisplayMessage(Table tableMessage, Duration duration) {
        this.tableMessage = tableMessage;
        this.displayDuration = duration;
    }

    /**
     * Возвращает период времени, в течении которого требуется отображать сообщение.
     * Сообщение будет автоматически скрыто по истечению периода.
     * @return период времени отображения сообщения или null, если его требуется отображать до прихода следующего сообщения или очистки дисплея.
     */
    public Duration getDisplayDuration() {
        return displayDuration;
    }

    /**
     * Устанавливает период времени, в течении которого требуется отображать сообщение.
     * Сообщение будет автоматически скрыто по истечению периода.
     * @param displayDuration период времени отображения сообщения или null, если его требуется отображать до прихода следующего сообщения или очистки дисплея.
     */
    public void setDisplayDuration(Duration displayDuration) {
        this.displayDuration = displayDuration;
    }

    /**
     * Возвращает сообщение для отображения на дисплее покупателя.
     * @return сообщение для отображения на дисплее покупателя или null, если требуется отобразить таблицу (в таком случае метод {@link #getTableMessage()} не должен
     * быть null).
     */
    public CommunicationMessage getMessage() {
        return message;
    }

    /**
     * Устанавливает сообщение для отображения на дисплее покупателя.
     * @param message сообщение для отображения на дисплее покупателя или null, если требуется отобразить таблицу (в таком случае метод {@link #getTableMessage()} не дожен
     *                быть null).
     */
    public void setMessage(CommunicationMessage message) {
        this.message = message;
    }

    /**
     * Возвращает таблицу, которую следует отобразить покупателю.
     * @return таблица которую следует отобразить покупателю или null, если отображается иное сообщение (в этом случе метод {@link #getMessage()} не должен возвращать null).
     */
    public Table getTableMessage() {
        return tableMessage;
    }

    /**
     * Устанавливает таблицу, которую слеудует отобразить покупателю.
     * @param tableMessage таблица, которую следует отобразить покупателю или null, если отображается иное сообщение (в этом случае метод {@link #getMessage()} не должен
     *                     возвращать null).
     */
    public void setTableMessage(Table tableMessage) {
        this.tableMessage = tableMessage;
    }

    @Override
    public String toString() {
        return "CustomerDisplayMessage{" +
                "displayDuration=" + displayDuration +
                ", message=" + message +
                ", tableMessage=" + tableMessage +
                '}';
    }
}
